/***************************************************************************
 *   Copyright (C) 2004 by David Banz                                      *
 *   neko@netcologne.de                                                    *
 *   GPL'ed                                                                *
 ***************************************************************************/

#ifndef CFULLGUI_H
#define CFULLGUI_H

// internal name of this release
#define INTERNAL_DESIGNATION "Briegel T. Busch"
// version number parts - here 2.1.7, keep consistent with VERSION defined in config.h !
#define MAJOR 2
#define MINOR 1
#define PATCH 7

#include <qaccel.h>
#include <qdragobject.h>
#include <kprocess.h>

#include "TfxControlData.h"

class QString;
class CSplash;

// C++ includes
#include <iostream>

// local includes
#include "C_includes.h"

#ifdef HAVE_CONFIG_H
	// autogenerated
	#include <config.h>
#endif

/**
 * This class adds actual functionality to CRawGui.
 *
 * It contains all the slots and other functions necessary
 * to to control TFMX-PLAY via the widgets provided by
 * CRawGui.
 *
 * @short Core class to control TFMX-PLAY via CRawGui
 * @author David Banz
 * @see CRawGui
 */
class CFullGui : public CRawGui
{
	Q_OBJECT
	
	public:
		CFullGui(QWidget* parent=0, const char* name=0);
		virtual ~CFullGui();
	
		/// make sure application is closed when top widget is closed
		void closeEvent(QCloseEvent*);
	
		/// helper for the signal handler
		void signalHelper();
		
	protected:
		/// called when something is dragged into the widget
		void dragEnterEvent(QDragEnterEvent* e);
		/// called when something is dropped onto the widget
		void dropEvent(QDropEvent* e);
		
		CSplash* m_pSplashScreen;	// startup splash-screen
		
		bool m_bShowSplash;		// if true, show splash-screen
		
		KShellProcess* m_pProc;		// process for tfmx-play
		
		int m_nSong;			// subsong number
		int m_nOpen;			// file open? 0=no
		int m_nPlaying;			// file playing? 0=no
		int m_nFreq;			// frequency (44/32/22/16/11 kHz)
		int m_nBits;			// bits per sample (8/16)
		int m_nPreloadSubsong;		// subsong to preselect
		int m_nSeparation;		// channel separation
		
		QString	m_oBrowseDir;		// start dir for the file browser
		QString m_oPreloadSong;		// songfile to preload
		QString m_oTfmxFile;		// filename of the tfmx song (mdat.foo/tfmx.foo)
		
		QAccel* m_pAcc;			// accelerator for keyboard events
		
		// tool tips and other ui-text (for i18n)
		QString m_oTipFileBar;
		QString m_oTipSongLcd;
		QString m_oTipFreqLcd;
		QString m_oTipBitLcd;
		QString m_pTipStatusBar[2];	//  [0]=stopped, [1]=playing
		QString m_oTipPlayButton;
		QString m_oTipStopButton;
		QString m_oTipOpenButton;
		QString m_oTipQuitButton;
		QString m_oTipUpButton;
		QString m_oTipDownButton;
		QString m_oTipEightButton;
		QString m_oTipSixteenButton;
		QString m_oTipHiButton;
		QString m_oTipLoButton;
		QString m_oTipHelpButton;
		QString m_oTipAboutButton;
		QString m_oTipToolTipCheck;
		QString m_pAboutStr[4];		// translations for "about"-box
		QString m_pNoTfxStr[4];		// translations for "no tfmx-play error"-box
		QString m_pTipSeparation[3];	// [0]=mono, [1]=headphones, [2]=stereo
		QString m_pShowSeparation[3];	// same here
		QString m_oTipMonoButton;
		QString m_oTipHeadphonesButton;
		QString m_oTipStereoButton;
		QString m_oTipOversampling;
		QString m_oTipGemX;
		QString m_oTipDangerFreak;
	
		/**
		* This turns on tooltips and unchecks the checkbox.
		*
		* @see #tipOff
		* @see #toggleCheckTip
		* @see #addTipBulk
		*/
		void tipOn();
	
		/**
		* This turns off tooltips and unchecks the checkbox.
		*
		* @see #tipOn
		* @see #toggleCheckTip
		* @see #removeTipBulk
		*/
		void tipOff();
	
		/**
		*	This updated the three LCDs.
		*
		* @see #playCore
		*
		*/
		void updateLcd();
		
		/**
		* This sets the song, tfmxFile, freq, bits, etc.,
		* updates the display and, if /dev/dsp is free, invokes TFMX-PLAY.
		*
		* @see #updateLcd
		*/
		void playCore();
	
		/**
		* This stops the player, neccessary display updates are also made.
		*
		* @see #stopTfx
		*/
		void stopCore();
	
		/**
		* This reads the configuration, determines whether to show the splash-screen,
		* sets up preloading of song from previous session
		*
		* @see #saveConfig
		*/
		void readConfig();
	
		/**
		* This saves the configuration.
		*
		* @see #readConfig
		*/
		void saveConfig();
	
	// --------------- these are mainly used by the constructor -----------------
	
		/**
		* Check if tfmx-play can be executed, exit with error message otherwise.
		* This method does the i18n init for the error box on its own!
		*
		* @see #i18nInitBulk
		*/
		void checkTfx();
	
		/**
		* This creates an accelerator and adds all the keyboard shortcuts.
		*
		* @see #buttonInit
		*/
		void accInit(); 
	
		/**
		* This connects all the buttons to the various slots.
		*
		* @see #buttonInit
		*/
		void buttonInit();
	
		/**
		* This sets all the initial defaults for the player, just to be safe...
		*
		* @see #buttonInit
		*/
		void initParanoia();
	
		/**
		* This updates the display sets up the player, if a song is
		* preloaded via the config file or command-line argument.
		*
		* @see #readConfig
		* @see #evalComm
		*/
		void initPreload();
	
		/**
		* Here we initialize most of the translations for buttons,
		* tooltips, message-boxes, etc.
		*
		* @see #checkTfx
		*/
		void i18nInitBulk();
		
	// ------------------------ enter the slots -------------------------
	
	protected slots:
		/// Toggle oversampling on/off.
		void toggleOversampling();
		
		/// Toggle gemx hack on/off.
		void toggleGemX();
	
		/// Toggle danger freak hack on/off.
		void toggleDangerFreak();
	
		/// Toggle oversampling and button on/off.
		void toggleOsampButton();
		
		/// Toggle gemx hack and button on/off.
		void toggleGemXButton();
		
		/// Toggle danger freak hack and button on/off.
		void toggleDangerFreakButton();
	
		/**
		* Called to cleanup the KShellProcess after the player has been stopped,
		* by means of the stop button, for example.
		*/
		void done(KProcess*);
	
		/**
		* This slot stops the player, then quits the application.
		*
		* @see CRawGui
		* @see #stopTfx
		*/
		void quitTfx();
	
		/// Overloaded w/o paramter.
		void openTfx();
		
		/**
		* This slot stops the player, then saves the config.
		*
		* @see CRawGui
		* @see #playTfx
		* @see #saveConfig
		* @see #stopCore
		*/
		void stopTfx();
	
		/**
		* This slot increases song num. and calls stopTfx and playTfx if necessary.
		* Config is saved.
		*
		* @see CRawGui
		* @see #songDown
		* @see #playTfx
		* @see #stopTfx
		* @see #saveConfig
		*/
		void songUp();
	
		/**
		* This slot decreases song num. and calls stopTfx and playTfx if necessary.
		* Config is saved.
		*
		* @see CRawGui
		* @see #songUp
		* @see #playTfx
		* @see #stopTfx
		* @see #saveConfig
		*/
		void songDown();
	
		/**
		* This slot increases freq and calls stopTfx and playTfx if necessary.
		* Config is saved.
		*
		* @see CRawGui
		* @see #freqDown
		* @see #playTfx
		* @see #stopTfx
		* @see #saveConfig
		*/
		void freqUp();
		
		/**
		* This slot decreases freq and calls stopTfx and playTfx if necessary.
		* Config is saved.
		*
		* @see CRawGui
		* @see #freqUp
		* @see #playTfx
		* @see #stopTfx
		* @see #saveConfig
		*/
		void freqDown();
	
		/**
		* This slot switches to 16 bits and calls stopTfx and playTfx if necessary.
		* Config is saved.
		*
		* @see CRawGui
		* @see #bitsLo
		* @see #playTfx
		* @see #stopTfx
		* @see #saveConfig
		*/
		void bitsHi();
	
		/**
		* This slot switches to 8 bits and calls stopTfx and playTfx if necessary.
		* Config is saved.
		*
		* @see CRawGui
		* @see #bitsHi
		* @see #playTfx
		* @see #stopTfx
		* @see #saveConfig
		*/
		void bitsLo();
	
		/**
		* Set stereo output mode, stop/restart/save config works as usual.
		*
		* @see #setMono
		* @see #setHeadphones
		* @see #playTfx
		* @see #stopTfx
		* @see #saveConfig
		*/
		void setStereo();
		
		/**
		* Set headphone output mode, stop/restart/save config works as usual.
		* If you want to use headphones with a stereo soundcard,
		* this mode usually sounds better than reguar stereo.
		*
		* @see #setStereo
		* @see #setMono
		* @see #playTfx
		* @see #stopTfx
		* @see #saveConfig
		*/
		void setHeadphones();
		
		/**
		* Set mono output mode, stop/restart/save config works as usual.
		*
		* @see #setHeadphones
		* @see #setStereo
		* @see #playTfx
		* @see #stopTfx
		* @see #saveConfig
		*/
		void setMono();
	
		/**
		* This slot toggles tooltips on/off, depending on current status
		* of the checkbox. Config is saved.
		*
		* @see CRawGui
		* @see #toggleCheckTip
		* @see #tipOff
		* @see #tipOn
		* @see #saveConfig
		* @see #addTipBulk
		* @see #removeTipBulk
		*/
		void toggleTip();
	
		/**
		* This slot turns tooltips off if are on and vice versa.
		* The checkbox is (un)checked, config is saved.
		*
		* @see CRawGui
		* @see #toggleTip
		* @see #tipOff
		* @see #tipOn
		* @see #saveConfig
		*/
		void toggleCheckTip();
	
		/// This does the real work of adding all the tooltips, nothing else.
		void addTipBulk();
		
		/// This does the real work of removing all the tooltips, nothing else.
		void removeTipBulk();
			
		/**
		* This slot invokes the HTML online help.
		*
		* @see CRawGui
		* @see #about
		*/
		void help();
	
		/**
		* This slot shows the "about" box.
		* I just couldn't resist putting the new application
		* into the about box, it's just too "kawaii"!
		*
		* @see CRawGui
		* @see #help
		*/
		void about();
	
		/**
		* Returns path of smpl.-file for given mdat.-filepath,
		* if smpl.-file exists. Returns "" otherwise.
		*
		* @param mdatPath a string containing the path of the mdat. file
		*/
		QString samplePath(const QString& mdatPath);
	
		/**
		* Returns path of .sam-file for given .tfx-filepath,
		* if .sam-file exists. Returns "" otherwise.
		*
		* @param mdatPath a string containing the path of the .tfx file
		*/
		QString dosSamplePath(const QString& mdatPath);
	
		/**
		* This slot plays the current file, otherwise openTfx is called
		* to get a file. Config is saved.
		*
		* @see CRawGui
		* @see #openTfx
		* @see #playCore
		* @see #saveConfig
		*/
		void playTfx();
		
		/**
		* This slot opens a KFileDialog to get a new tfmxFile to play,
		* if fileName is 0. Otherwise, the path given in fileName is used instead.
		* Valid selections are played, config is saved.
		*
		* @param fileName is the path to the file to be opened.
		* @see CRawGui
		* @see #playTfx
		*/
		void openTfx(const QString& fileName);
	
	public slots:
	
		/**
		* This file sets the current file to fileName, if not playing.
		*
		* @param fileName is the path to the file to be opened.
		* @see #openTfx
		*/
		void setFile(const QString& fileName);
	
		/**
		* This slot sets the subsong to num, if not playing.
		*
		* @see #playTfx
		*/
		void setSong(int num);
	
		/**
		* This slot checks if a tfmx file has been loaded, if so, it starts playing.
		*
		* @see #playTfx
		*/
		void playStart();
};

#endif
